local L = LibStub( "AceLocale-3.0" ):GetLocale( "UnderHood" )
local UH = UnderHood
local AceEvent = LibStub( "AceEvent-3.0" )
local AceEvent2 = LibStub( "AceEvent-2.0" )
local Threat = LibStub( "Threat-1.0" )
local _G = _G

local Provider = {}
Provider.__index = Provider
Provider.name = "Threat"
Provider.title = L["Threat"]
Provider.supportedUnits = { "player" }
Provider.manageVisibility = true

	
function Provider:Create( bar )
	local self = {}

	setmetatable( self, Provider )
	AceEvent:Embed( self )

	self.bar = bar

	return self
end

function Provider:UnitSupported( unit )
	return unit == "player"
end

function Provider:IsBarVisible()
	return Threat:IsActive() and self.targetName
end

function Provider:Connect()
	AceEvent2:RegisterEvent( "Threat_ThreatUpdated", function( ... ) self:ThreatUpdated( ... ) end )

	self:RegisterEvent( "PLAYER_TARGET_CHANGED" )

	self:PLAYER_TARGET_CHANGED()
end

function Provider:Disconnect()
	self:UnregisterAllEvents()
	self:UnregisterAllMessages()

	AceEvent2:UnregisterEvent( "Threat_ThreatUpdated" )
end

function Provider:Update()
	local r,g,b = unpack( UH.db.profile.colors.unit.hostile )

	self.bar:SetColor( r, g, b, 1 )

	if Threat:IsActive() and self.targetName then
		self.bar:Show()
	else
		self.bar:Hide()
	end

	self:SetValue()
end

function Provider:ThreatUpdated( name, unit, hash, value )
	if not self.targetName then return end

	if unit == "player" or (hash and Threat:UnitIsUnit( self.targetName, hash )) then
	    self:SetValue()
	end
end

function Provider:PLAYER_TARGET_CHANGED()
	self.targetName = nil

	if UnitCanAttack( "player", "target" ) then
	    self.targetName = UnitName( "target" )
	end

	self:Update()
end

function Provider:SetValue()
	local threat, maxThreat

	if Threat:IsActive() and self.targetName then
		threat = Threat:GetThreat( UnitName( "player" ), self.targetName )
		maxThreat = Threat:GetMaxThreatOnTarget( self.targetName )
	end

	if not( maxThreat ) or maxThreat == 0 then
		threat = 0
		maxThreat = 1
	else
		threat = threat/maxThreat
	end

	self.bar:SetValue( threat )
end

UnderHood_Bars:RegisterProvider( Provider )
